/*************************************************************
 * Author:	wangjiangyuan (wangjiangyuan@agora.io)
 * Date	 :	Oct 21th, 2020
 * Module:	Agora SD-RTN SDK RTC C API demo application.
 *
 *
 * This is a part of the Agora RTC Service SDK.
 * Copyright (C) 2020 Agora IO
 * All rights reserved.
 *
 *************************************************************/

#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <unistd.h>

#include "file_parser.h"
#include "file_parser_priv.h"

#define ADTS_HEADER_SIZE (7)

typedef struct {
	int32_t data_offset_;
	int32_t data_size_;
	uint8_t *data_buffer_;
	int     fd_;
} ctx_t;

static int32_t pcm_open(media_parser_t *h, const char *path)
{
	int32_t rval = -1;

	int32_t fd = -1;
	do {
		struct stat sb;
		void *mapped;

		fd = open(path, O_RDONLY);
		if (fd < 0) {
			LOGD("parser: invalid fd, path=%s", path);
			break;
		}
        LOGI("Open audio file %s successfully", path);

		if ((fstat(fd, &sb)) == -1) {
			break;
		}

		mapped = mmap(NULL, sb.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
		if (mapped == (void *)-1) {
			break;
		}

		ctx_t *p_ctx = (ctx_t *)malloc(sizeof(ctx_t));
		if (!p_ctx) {
			break;
		}
		p_ctx->data_size_ = sb.st_size;
		p_ctx->data_buffer_ = (uint8_t *)mapped;
		p_ctx->data_offset_ = 0;
		p_ctx->fd_ = fd;

		h->p_ctx = (void *)p_ctx;
		rval = 0;
	} while (0);

	return rval;
}

static int32_t pcm_obtain_frame(media_parser_t *h, frame_t *p_frame)
{
	int32_t rval = -1;

	do {
		ctx_t *p_ctx = (ctx_t *)h->p_ctx;
		if (!p_ctx) {
			LOGD("parser: invalid ctx");
			break;
		}

		// Check data offset and rewind to the file start if necessary
		if (p_ctx->data_offset_ + ADTS_HEADER_SIZE > p_ctx->data_size_) {
			// p_ctx->data_offset_ = 0;
            rval = -2;
            break;
		}

		int32_t frameSizeDuration = 20;

		p_frame->type = h->type;
		p_frame->ptr = p_ctx->data_buffer_ + p_ctx->data_offset_;

		p_frame->u.audio.sampleRateHz = h->parser_cfg.u.audio_cfg.sampleRateHz;
		p_frame->u.audio.samplesPerChannel =
				p_frame->u.audio.sampleRateHz * frameSizeDuration / 1000;
		p_frame->u.audio.numberOfChannels = h->parser_cfg.u.audio_cfg.numberOfChannels;

		p_frame->len = (p_frame->u.audio.samplesPerChannel * p_frame->u.audio.numberOfChannels * sizeof(int16_t));
		p_ctx->data_offset_ += p_frame->len;

		rval = 0;
	} while (0);

	return rval;
}

static int32_t pcm_reset(media_parser_t *h)
{
	int32_t rval = -1;
	do {
		ctx_t *p_ctx = (ctx_t *)h->p_ctx;
		if (!p_ctx) {
			break;
		}

		p_ctx->data_offset_ = 0;
		rval = 0;
	} while (0);

	return rval;
}

static int32_t pcm_close(media_parser_t *h)
{
	int32_t rval = -1;
	do {
		ctx_t *p_ctx = (ctx_t *)h->p_ctx;
		if (!p_ctx) {
			break;
		}

		if (p_ctx->data_buffer_) {
			munmap(p_ctx->data_buffer_, p_ctx->data_size_);
			p_ctx->data_buffer_ = NULL;
			p_ctx->data_size_ = 0;
		}

		if (p_ctx->fd_) {
			close(p_ctx->fd_);
		}

		free(p_ctx);
		h->p_ctx = NULL;
		rval = 0;
	} while (0);

	return rval;
}

media_parser_t audio_parser_pcm = {
	.codec = AUDIO_CODEC_PCM,
	.p_ctx = NULL,

	.open = pcm_open,
	.obtain_frame = pcm_obtain_frame,
	.reset = pcm_reset,
	.close = pcm_close,
    .parser_cfg = {
        .u = {
            .audio_cfg = {
                .sampleRateHz = -1,
                .numberOfChannels = -1,
            }
        }
    }
};
